@@ -78,6 +78,7 @@ struct _AppIndicatorPrivate {
 	GtkWidget            *sec_activate_target;
 	gboolean              sec_activate_enabled;
 	guint32               ordering_index;
+	gchar *               title;
 	gchar *               label;
 	gchar *               label_guide;
 	gchar *               accessible_desc;
@@ -127,7 +128,8 @@ enum {
 	PROP_LABEL,
 	PROP_LABEL_GUIDE,
 	PROP_ORDERING_INDEX,
-	PROP_DBUS_MENU_SERVER
+	PROP_DBUS_MENU_SERVER,
+	PROP_TITLE
 };
 
 /* The strings so that they can be slowly looked up. */
@@ -144,6 +146,7 @@ enum {
 #define PROP_LABEL_GUIDE_S           "label-guide"
 #define PROP_ORDERING_INDEX_S        "ordering-index"
 #define PROP_DBUS_MENU_SERVER_S      "dbus-menu-server"
+#define PROP_TITLE_S                 "title"
 
 /* Private macro, shhhh! */
 #define APP_INDICATOR_GET_PRIVATE(o) \
@@ -251,7 +254,7 @@ app_indicator_class_init (AppIndicatorClass *klass)
                                                               G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_CONSTRUCT_ONLY));
 
 	/**
-	 * AppIndicator:
+	 * AppIndicator:status:
 	 *
 	 * Whether the indicator is shown or requests attention. Defaults to
 	 * 'Passive'.
@@ -410,6 +413,21 @@ app_indicator_class_init (AppIndicatorClass *klass)
 	                                                     "DBusmenu server which is available for testing the application indicators.",
 	                                                     DBUSMENU_TYPE_SERVER,
 	                                                     G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	/**
+	 * AppIndicator:title:
+	 *
+	 * Provides a way to refer to this application indicator in a human
+	 * readable form.  This is used in the Unity desktop in the HUD as
+	 * the first part of the menu entries to distinguish them from the
+	 * focused application's entries.
+	 */
+	g_object_class_install_property(object_class,
+	                                PROP_TITLE,
+	                                g_param_spec_string (PROP_TITLE_S,
+	                                                     "Title of the application indicator",
+	                                                     "A human readable way to refer to this application indicator in the UI.",
+	                                                     NULL,
+	                                                     G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/* Signals */
 
@@ -575,6 +593,7 @@ app_indicator_init (AppIndicator *self)
 	priv->menu = NULL;
 	priv->menuservice = NULL;
 	priv->ordering_index = 0;
+	priv->title = NULL;
 	priv->label = NULL;
 	priv->label_guide = NULL;
 	priv->label_change_idle = 0;
@@ -716,6 +735,11 @@ app_indicator_finalize (GObject *object)
 		priv->icon_theme_path = NULL;
 	}
 	
+	if (priv->title != NULL) {
+		g_free(priv->title);
+		priv->title = NULL;
+	}
+
 	if (priv->label != NULL) {
 		g_free(priv->label);
 		priv->label = NULL;
@@ -830,20 +854,55 @@ app_indicator_set_property (GObject * object, guint prop_id, const GValue * valu
 		  gchar * oldlabel = priv->label;
 		  priv->label = g_value_dup_string(value);
 
-		  if (g_strcmp0(oldlabel, priv->label) != 0) {
-		    signal_label_change(APP_INDICATOR(object));
-		  }
-
 		  if (priv->label != NULL && priv->label[0] == '\0') {
 		  	g_free(priv->label);
 			priv->label = NULL;
 		  }
 
+		  if (g_strcmp0(oldlabel, priv->label) != 0) {
+		    signal_label_change(APP_INDICATOR(object));
+		  }
+
 		  if (oldlabel != NULL) {
 		  	g_free(oldlabel);
 		  }
 		  break;
 		}
+		case PROP_TITLE: {
+		  gchar * oldtitle = priv->title;
+		  priv->title = g_value_dup_string(value);
+
+		  if (priv->title != NULL && priv->title[0] == '\0') {
+		  	g_free(priv->title);
+			priv->title = NULL;
+		  }
+
+		  if (g_strcmp0(oldtitle, priv->title) != 0 && self->priv->connection != NULL) {
+			GError * error = NULL;
+
+			g_dbus_connection_emit_signal(self->priv->connection,
+										  NULL,
+										  self->priv->path,
+										  NOTIFICATION_ITEM_DBUS_IFACE,
+										  "NewTitle",
+										  NULL,
+										  &error);
+
+			if (error != NULL) {
+				g_warning("Unable to send signal for NewTitle: %s", error->message);
+				g_error_free(error);
+			}
+		  }
+
+		  if (oldtitle != NULL) {
+		  	g_free(oldtitle);
+		  }
+
+		  if (priv->status_icon != NULL) {
+		  	gtk_status_icon_set_title(priv->status_icon, priv->title ? priv->title : "");
+		  }
+		  break;
+		}
 		case PROP_LABEL_GUIDE: {
 		  gchar * oldguide = priv->label_guide;
 		  priv->label_guide = g_value_dup_string(value);
@@ -867,11 +926,8 @@ app_indicator_set_property (GObject * object, guint prop_id, const GValue * valu
 		  break;
 
 		case PROP_DBUS_MENU_SERVER:
-			if (priv->menuservice != NULL) {
-				g_object_unref (priv->menuservice);
-			}
-			gpointer val = g_value_dup_object(value);
-			priv->menuservice = DBUSMENU_SERVER(val);
+			g_clear_object (&priv->menuservice);
+			priv->menuservice = DBUSMENU_SERVER (g_value_dup_object(value));
 			break;
 
         default:
@@ -956,6 +1012,10 @@ app_indicator_get_property (GObject * object, guint prop_id, GValue * value, GPa
 			g_value_set_object(value, priv->menuservice);
 			break;
 
+		case PROP_TITLE:
+			g_value_set_string(value, priv->title);
+			break;
+
         default:
           G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
           break;
@@ -1062,6 +1122,8 @@ bus_get_prop (GDBusConnection * connection, const gchar * sender, const gchar *
 		return g_variant_new_string(priv->icon_name ? priv->icon_name : "");
 	} else if (g_strcmp0(property, "AttentionIconName") == 0) {
 		return g_variant_new_string(priv->attention_icon_name ? priv->attention_icon_name : "");
+	} else if (g_strcmp0(property, "Title") == 0) {
+		return g_variant_new_string(priv->title ? priv->title : "");
 	} else if (g_strcmp0(property, "IconThemePath") == 0) {
 		return g_variant_new_string(priv->icon_theme_path ? priv->icon_theme_path : "");
 	} else if (g_strcmp0(property, "Menu") == 0) {
@@ -1449,7 +1511,11 @@ fallback (AppIndicator * self)
 {
 	GtkStatusIcon * icon = gtk_status_icon_new();
 
-	gtk_status_icon_set_title(icon, app_indicator_get_id(self));
+	gtk_status_icon_set_name(icon, app_indicator_get_id(self));
+	const gchar * title = app_indicator_get_title(self);
+	if (title != NULL) {
+		gtk_status_icon_set_title(icon, title);
+	}
 	
 	g_signal_connect(G_OBJECT(self), APP_INDICATOR_SIGNAL_NEW_STATUS,
 		G_CALLBACK(status_icon_status_wrapper), icon);
@@ -1521,28 +1587,58 @@ static void
 status_icon_changes (AppIndicator * self, gpointer data)
 {
 	GtkStatusIcon * icon = GTK_STATUS_ICON(data);
-	gchar *longname = NULL;
 
+	/* add the icon_theme_path once if needed */
+	GtkIconTheme *icon_theme = gtk_icon_theme_get_default();
+	if (self->priv->icon_theme_path != NULL) {
+		gchar **path;
+		gint n_elements, i;
+		gboolean found=FALSE;
+		gtk_icon_theme_get_search_path(icon_theme, &path, &n_elements);
+		for (i=0; i< n_elements || path[i] == NULL; i++) {
+			if(g_strcmp0(path[i], self->priv->icon_theme_path) == 0) {
+				found=TRUE;
+				break;
+			}
+		}
+		if(!found) {
+			gtk_icon_theme_append_search_path(icon_theme, self->priv->icon_theme_path);
+		}
+		g_strfreev (path);
+	}
+
+	const gchar * icon_name = NULL;
 	switch (app_indicator_get_status(self)) {
 	case APP_INDICATOR_STATUS_PASSIVE:
-		longname = append_panel_icon_suffix(app_indicator_get_icon(self));
+		/* hide first to avoid that the change is visible to the user */
 		gtk_status_icon_set_visible(icon, FALSE);
-		gtk_status_icon_set_from_icon_name(icon, longname);
+		icon_name = app_indicator_get_icon(self);
 		break;
 	case APP_INDICATOR_STATUS_ACTIVE:
-		longname = append_panel_icon_suffix(app_indicator_get_icon(self));
-		gtk_status_icon_set_from_icon_name(icon, longname);
+		icon_name = app_indicator_get_icon(self);
 		gtk_status_icon_set_visible(icon, TRUE);
 		break;
 	case APP_INDICATOR_STATUS_ATTENTION:
-		longname = append_panel_icon_suffix(app_indicator_get_attention_icon(self));
-		gtk_status_icon_set_from_icon_name(icon, longname);
+		/* get the _attention_ icon here */
+		icon_name = app_indicator_get_attention_icon(self);
 		gtk_status_icon_set_visible(icon, TRUE);
 		break;
 	};
 
-	if (longname) {
-		g_free(longname);
+	if (icon_name != NULL) {
+		if (g_file_test(icon_name, G_FILE_TEST_EXISTS)) {
+			gtk_status_icon_set_from_file(icon, icon_name);
+		} else {
+			gchar *longname = append_panel_icon_suffix(icon_name);
+
+			if (longname != NULL && gtk_icon_theme_has_icon (icon_theme, longname)) {
+				gtk_status_icon_set_from_icon_name(icon, longname);
+			} else {
+				gtk_status_icon_set_from_icon_name(icon, icon_name);
+			}
+
+			g_free(longname);
+		}
 	}
 
 	return;
@@ -2094,6 +2190,36 @@ app_indicator_set_secondary_activate_target (AppIndicator *self, GtkWidget *menu
 	g_signal_connect(menuitem, "parent-set", G_CALLBACK(sec_activate_target_parent_changed), self);
 }
 
+/**
+ * app_indicator_set_title:
+ * @self: The #AppIndicator
+ * @title: (allow-none): Title of the app indicator
+ *
+ * Sets the title of the application indicator, or how it should be referred
+ * in a human readable form.  This string should be UTF-8 and localized as it
+ * expected that users will set it.
+ *
+ * In the Unity desktop the most prominent place that this is show will be
+ * in the HUD.  HUD listings for this application indicator will start with
+ * the title as the first part of the line for the menu items.
+ *
+ * Setting @title to %NULL removes the title.
+ *
+ * Since: 0.5
+ *
+ */
+void
+app_indicator_set_title (AppIndicator *self, const gchar * title)
+{
+	g_return_if_fail (IS_APP_INDICATOR (self));
+
+	g_object_set(G_OBJECT(self),
+	             PROP_TITLE_S, title == NULL ? "": title,
+	             NULL);
+
+	return;
+}
+
 /**
  * app_indicator_get_id:
  * @self: The #AppIndicator object to use
@@ -2222,6 +2348,27 @@ app_indicator_get_attention_icon_desc (AppIndicator *self)
   return self->priv->att_accessible_desc;
 }
 
+/**
+ * app_indicator_get_title:
+ * @self: The #AppIndicator object to use
+ *
+ * Gets the title of the application indicator.  See the function
+ * app_indicator_set_title() for information on the title.
+ *
+ * Return value: The current title.
+ *
+ * Since: 0.5
+ *
+ */
+const gchar *
+app_indicator_get_title (AppIndicator *self)
+{
+	g_return_val_if_fail (IS_APP_INDICATOR (self), NULL);
+
+	return self->priv->title;
+}
+
+
 /**
  * app_indicator_get_menu:
  * @self: The #AppIndicator object to use
